"""
ngm
==========
License: BSD, see LICENSE for more details.
"""
# PYTHON_ARGCOMPLETE_OK
import argcomplete
import argparse
import json

from tabulate import tabulate


def main():
    parser = argparse.ArgumentParser(description='CLI tool for managing NGINX configuration.',
                                     prog='ngm')

    subparsers = parser.add_subparsers(help='Command to run', dest="command")

    parser_modules = subparsers.add_parser('modules', help='Manage dynamic NGINX modules')
    parser_modules.add_argument('action', nargs='?', default='list', choices=['list'],
                                help='An action for managing dynamic NGINX modules')

    parser_modules = subparsers.add_parser('sites', help='Manage NGINX websites')
    parser_modules.add_argument('action', nargs='?', default='list', choices=['list'],
                                help='An action for managing NGINX modules')

    argcomplete.autocomplete(parser)
    args = parser.parse_args()

    if args.command == 'modules' and args.action == 'list':
        import os

        # load modules by soname
        modules_json = os.path.dirname(os.path.realpath(__file__)) + '/modules.json'
        with open(modules_json) as json_file:
            modules_data = json.load(json_file)

        modules_dir = '/etc/nginx/modules'
        module_files = [f for f in os.listdir(modules_dir)
                        if f.endswith('.so') and not f.endswith('-debug.so')]
        # TODO add update column (have to consult RPM db)
        all_modules = [['Package Name', 'Feature Summary', 'Load Directive', 'Reference URL']]
        os_codename = None  # e.g. el7
        # get distro
        if module_files:
            import distro
            d = distro.info()
            if d['id'] in ['centos', 'rhel', 'oracle']:
                os_codename = f"el{d['version_parts']['major']}"

        for module_file in module_files:
            package_name = '-'
            feature_summary = ''
            url = ''
            soname = module_file.replace('.so', '')
            if soname in modules_data['by_soname']:
                handle = modules_data['by_soname'][soname]
                package_name = f'nginx-module-{handle}'
                module_data = modules_data['by_handle'][handle]
                feature_summary = module_data['summary']
                url = f"https://github.com/{module_data['repo']}"
                if 'ref' in module_data and os_codename in module_data['ref']:
                    url = module_data['ref'][os_codename]
            else:
                print(f'{soname} not in modules_data')
            all_modules.append([package_name, feature_summary, f"load_module {module_file};", url])
        print(
            tabulate(all_modules, headers="firstrow", tablefmt='psql')
        )
